 Program Three_Dimensional_DC_and_IP_Forward_Modelling

    Implicit None
!==================================================================================================================
     ! Subroutine for computation of the DC electrical potential based on
     ! Local Normal Vectors which are produced for each element
    INTERFACE
    Subroutine Local_Normal_Potential(Number_of_Nodes,Number_of_Cells,ele_normal,&
    &E_CON,A_CON,current,Node_coordinates,SL,integral_order,Cell_Node_Numbers,Potential_Electrod,COG)      
      Integer, Allocatable, Dimension(:,:), INTENT(IN) :: Cell_Node_Numbers
      DOUBLE PRECISION, Allocatable, Dimension(:,:), INTENT(IN) :: ele_normal
      DOUBLE PRECISION, Allocatable, Dimension(:,:), INTENT(IN) :: Node_coordinates
      Real, Allocatable, Dimension(:), INTENT(IN) :: SL
      Real, Allocatable, Dimension(:,:), INTENT(IN) :: Potential_Electrod
      DOUBLE PRECISION, Allocatable, Dimension(:,:), INTENT(IN) :: COG
      Integer, INTENT(IN) :: Number_of_Nodes,Number_of_Cells,integral_order
      Real, INTENT(IN) :: E_CON,A_CON,current
    END SUBROUTINE  Local_Normal_Potential
    END INTERFACE

!==================================================================================================================
     ! Subroutine for Computation of the gravity center of each triangular elements
    INTERFACE
    SUBROUTINE Center_of_Gravity(Number_of_Cells,Cell_Node_Numbers,Node_coordinates,COG)
        Integer, Allocatable, Dimension(:,:), INTENT(IN) :: Cell_Node_Numbers
        DOUBLE PRECISION, Allocatable, Dimension(:,:), INTENT(IN) :: Node_coordinates
        DOUBLE PRECISION, Allocatable, Dimension(:,:), INTENT(OUT) :: COG

        Integer, INTENT(IN) :: Number_of_Cells
    END SUBROUTINE Center_of_Gravity
    END INTERFACE
!==================================================================================================================
     !Subroutine for Computation of the normal vectors of the sphere anomaly nodes
    INTERFACE
    Subroutine Sphere_Nodal_Normal(COG,Sphere,Number_of_Cells,S_Normal)

     DOUBLE PRECISION, Allocatable, Dimension(:,:), INTENT(IN) :: COG
     Real, Allocatable, Dimension(:), INTENT(IN) :: Sphere
     DOUBLE PRECISION, Allocatable, Dimension(:,:), INTENT(OUT) :: S_Normal
     Integer, INTENT(IN) :: Number_of_Cells
    END SUBROUTINE Sphere_Nodal_Normal
    END INTERFACE
!==================================================================================================================
     Integer :: i,j,k,a1,a2,a3,anomalytype,Number_of_Electrodes
     Integer :: Number_of_Nodes,Number_of_Cells,integral_order
    
     Real :: E_CON,A_CON,current
     Real :: Chargeability
     Real :: tr1,tr2,tr3,tr4
     Real :: Xc,Yc,Zc,Radius

     Integer, Allocatable, Dimension(:,:) :: Cell_Node_Numbers  
     DOUBLE PRECISION, Allocatable, Dimension(:,:) :: Node_coordinates
     DOUBLE PRECISION, Allocatable, Dimension(:,:) :: ele_normal     
     DOUBLE PRECISION, Allocatable, Dimension(:,:) :: S_Normal
     Real, Allocatable, Dimension(:) :: SL       ! Source Location     
     Real, Allocatable, Dimension(:,:) :: Potential_Electrod     
     Real, Allocatable, Dimension(:,:) :: Pole1
     Real, Allocatable, Dimension(:,:) :: Pole2
     Real, Allocatable, Dimension(:) :: Sphere
     Real, Allocatable, Dimension(:) :: dipole_center
     DOUBLE PRECISION, Allocatable, Dimension(:) :: p1
     DOUBLE PRECISION, Allocatable, Dimension(:) :: p2
     DOUBLE PRECISION, Allocatable, Dimension(:,:) :: COG   

    !Print*, "Please Enter the Number of Nodes"
    !Read*, Number_of_Nodes
    !Print*, "Please Enter the Number of Cells"
    !Read*, Number_of_Cells
    Number_of_Nodes = 762
    Number_of_Cells = 1520
    Number_of_Electrodes = 251

    Allocate(Node_coordinates(Number_of_Nodes,4))
    Allocate(Cell_Node_Numbers(Number_of_Cells,4))
    Allocate(ele_normal(Number_of_Cells,4))
    Allocate(S_Normal(Number_of_Cells,4))
    Allocate(SL(3))
    Allocate(Potential_Electrod(251,3))
    Allocate(Pole1(251,3))
    Allocate(Pole2(251,3))
    Allocate(Sphere(4))
    Allocate(p1(251))
    Allocate(p2(251))
    Allocate(dipole_center(251))
    Allocate(COG(Number_of_Cells,4))
!==================================================================================================================
    ! Reading the conductivities of the half-space and anomaly
    !Print*, "Please Enter the the conductivities of the half-space and anomaly respectively"
    !Read*, E_CON,A_CON
    E_CON = 0.0001
    A_CON = 1.0
    !Chargeability = 0.1
    !A_CON = 0.001 * (1 - Chargeability)
!==================================================================================================================
    ! Reading the amount of the electrical current injecting into the earth through currenr electrodes
    !Print*, "Please Enter the amount of electrical current "
    !Read*, current
    current = 1
!==================================================================================================================
    ! Reading the order of numerical integration
    !Print*, "Please Enter the order of the numerical integration (Options are : 2,3,4,5,6) "
    !Read*, integral_order
    integral_order = 6
!==================================================================================================================
    ! Reading the coordinates of the Nodes Global Number and their coordinates
    Open(10, file = 'surf.node')
    Read(10,*)
    Do i=1,Number_of_Nodes
            Read(10,*)  Node_Coordinates(i,1),Node_Coordinates(i,3),Node_Coordinates(i,2),Node_Coordinates(i,4)
    End Do
!==================================================================================================================
    ! Reading the the number of the cells and the global number of the nodes in each cells
    Open(20, file = 'surf.ele')
    Read(20,*)
    Do i=1,Number_of_Cells
            
            Read(20,*)  Cell_Node_Numbers(i,1),Cell_Node_Numbers(i,2),Cell_Node_Numbers(i,3),Cell_Node_Numbers(i,4),tr1,tr2,tr3
    End Do
!==================================================================================================================
    ! Reading the the outward normal vectors of each element
    Open(30, file = 'surf.ele')
    Read(30,*)
    Do i=1,Number_of_Cells
            Read(30,*)  ele_normal(i,1),tr1,tr2,tr3,ele_normal(i,3),ele_normal(i,2),ele_normal(i,4)
    End Do
!==================================================================================================================
    ! Reading the location of the electric current source
    Open(45, file = 'Source_Location.txt')
    Read(45,*)  SL(1),SL(2),SL(3)    
!==================================================================================================================
    ! Reading the location of the Potential Electrod
    Open(50, file = 'electrod.txt')
    Read(50,*)
    Read(50,*)
    Do i=1,Number_of_Electrodes
    Read(50,*)  Potential_Electrod(i,1),Potential_Electrod(i,2),Potential_Electrod(i,3),tr1,tr2,tr3
    End Do
    Open(5000, file = 'New_electrod.txt')
    Do i=1,Number_of_Electrodes
    Potential_Electrod(i,2) = 350
    Write(5000,*)  Potential_Electrod(i,1),"    ",Potential_Electrod(i,2),"    ",Potential_Electrod(i,3)
    End Do

!==================================================================================================================
    !Reading the coordinates of the Sphere anomaly
    Open(55, file = 'Sphere.txt')
    Read(55,*)  Sphere(1),Sphere(2),Sphere(3),Sphere(4)
    Xc = Sphere(1)
    Yc = Sphere(2)
    Zc = Sphere(3)
    Radius = Sphere(4)
!==================================================================================================================        
    Call Center_of_Gravity(Number_of_Cells,Cell_Node_Numbers,Node_coordinates,COG)

    Call Sphere_Nodal_Normal(COG,Sphere,Number_of_Cells,S_Normal)

    Call Local_Normal_Potential(Number_of_Nodes,Number_of_Cells,S_Normal,&
    &E_CON,A_CON,current,Node_coordinates,SL,integral_order,Cell_Node_Numbers,Potential_Electrod,COG)   

!==================================================================================================================
    ! Producing the vtk file to illustrate the normal vectors
    Open(55000, file = 'normal_vector.vtk')
    Open(66000, file = 'normal_node.txt')
    Write(55000,*)  "# vtk DataFile Version 2.0"
    Write(55000,*)  "Vector example"
    Write(55000,*)  "ASCII"
    Write(55000,*)  "DATASET UNSTRUCTURED_GRID"
    Write(55000,*)  "POINTS       ",Number_of_Cells," float"
    Do i=1,Number_of_Cells
        Write(55000,*) COG(i,2),"    ",COG(i,3),"    ",COG(i,4)       
    End Do
    Do i=1,Number_of_Nodes
     Write(66000,*) Node_Coordinates(i,1),"    ",Node_Coordinates(i,2),"    ",Node_Coordinates(i,3),"    ",Node_Coordinates(i,4)
    End Do
    Write(55000,*) "POINT_DATA  ", Number_of_Cells
    Write(55000,*) "VECTORS vectors float  "
    Do i=1,Number_of_Cells
        Write(55000,*) ele_normal(i,4),"    ",ele_normal(i,3),"    ",ele_normal(i,4)
    End Do
!==================================================================================================================
     
    End Program Three_Dimensional_DC_and_IP_Forward_Modelling