Subroutine Local_Normal_Potential(Number_of_Nodes,Number_of_Cells,ele_normal,&
    &E_CON,A_CON,current,Node_coordinates,SL,integral_order,Cell_Node_Numbers,Potential_Electrod,COG)

    Implicit None

!==================================================================================================================
    ! Subroutine for Computation of the normal gradient of source potential (Right hand side of the final matrix equation)
    INTERFACE
    SUBROUTINE Normal_Gradient_Source_Potential(E_CON,A_CON,current,Number_of_Cells,SL,ele_normal,COG,RHS)
    Real, Allocatable, Dimension(:), INTENT(IN) :: SL        
    DOUBLE PRECISION, Allocatable, Dimension(:,:), INTENT(IN) :: COG
    DOUBLE PRECISION, Allocatable, Dimension(:,:), INTENT(IN) :: ele_normal
    DOUBLE PRECISION, Allocatable, Dimension(:), INTENT(OUT) :: RHS

    Integer, INTENT(IN) :: Number_of_Cells
    Real, INTENT(IN) :: E_CON,A_CON,current
    END SUBROUTINE Normal_Gradient_Source_Potential
    END INTERFACE
!==================================================================================================================
    ! Subroutine for Computation of the Left hand side of the final matrix equation
    INTERFACE
    SUBROUTINE Local_Coefficient_Matrix(E_CON,A_CON,integral_order,Number_of_Nodes,Number_of_Cells,&
        &Cell_Node_Numbers,ele_normal,Node_coordinates,COG,Local_LHS)
      Integer, Allocatable, Dimension(:,:), INTENT(IN) :: Cell_Node_Numbers
      DOUBLE PRECISION, Allocatable, Dimension(:,:), INTENT(IN) :: Node_coordinates            
      DOUBLE PRECISION, Allocatable, Dimension(:,:), INTENT(IN) :: ele_normal
      DOUBLE PRECISION, Allocatable, Dimension(:,:), INTENT(IN) :: COG
      DOUBLE PRECISION, Allocatable, Dimension(:,:), INTENT(OUT) :: Local_LHS
     ! LHS : Left Hand Side Matrix (Coessicien Matrix in LHS C = RHS)
     Integer, INTENT(IN) :: Number_of_Nodes,Number_of_Cells,integral_order
      Real, INTENT(IN) :: E_CON,A_CON
    END SUBROUTINE Local_Coefficient_Matrix
    END INTERFACE
!==================================================================================================================
    !Subroutine for Computation of the DC potential
    INTERFACE
    SUBROUTINE DC_Potential(E_CON,current,integral_order,Cell_Electric_Charge,Number_of_Nodes,Number_of_Cells,&
    &SL,PEL,Node_coordinates,Cell_Node_Numbers,ele_normal,Half_Space_Potential,Anomaly_Potential,Potential)

    !DCP_ : DC Potential, The first letters of the subroutine's name
      Integer, Allocatable, Dimension(:,:), INTENT(IN) :: Cell_Node_Numbers
      DOUBLE PRECISION, Allocatable, Dimension(:,:), INTENT(IN) :: Node_coordinates      
      DOUBLE PRECISION, Allocatable, Dimension(:), INTENT(IN) :: Cell_Electric_Charge
      Real, Allocatable, Dimension(:), INTENT(IN) :: SL
      Real, Allocatable, Dimension(:), INTENT(IN) :: PEL
      DOUBLE PRECISION, Allocatable, Dimension(:,:), INTENT(IN) :: ele_normal
      Integer, INTENT(IN) :: Number_of_Nodes,Number_of_Cells,integral_order
      Real, INTENT(IN) :: E_CON,current
      DOUBLE PRECISION, INTENT(OUT) :: Potential
      DOUBLE PRECISION, INTENT(OUT) :: Half_Space_Potential
      DOUBLE PRECISION, INTENT(OUT) :: Anomaly_Potential
    END SUBROUTINE DC_Potential
    END INTERFACE
!==================================================================================================================     
      Integer, Allocatable, Dimension(:,:), INTENT(IN) :: Cell_Node_Numbers
      DOUBLE PRECISION, Allocatable, Dimension(:,:), INTENT(IN) :: ele_normal
      DOUBLE PRECISION, Allocatable, Dimension(:,:), INTENT(IN) :: Node_coordinates
      Real, Allocatable, Dimension(:), INTENT(IN) :: SL
      Real, Allocatable, Dimension(:,:), INTENT(IN) :: Potential_Electrod
      Integer, INTENT(IN) :: Number_of_Nodes,Number_of_Cells,integral_order
      DOUBLE PRECISION, Allocatable, Dimension(:,:), INTENT(IN) :: COG
      Real, INTENT(IN) :: E_CON,A_CON,current
!==================================================================================================================
    Integer :: ignp1
    DOUBLE PRECISION :: rgnp1,rgnp2,rgnp3,rgnp4
    ! ignp :: integer variable in the subroutine Global Normal Potential
    ! rgnp :: real variable in the subroutine Global Normal Potential
    Integer :: i,j,l,k
    Integer :: NRHS,LDA,LDB,INFO
    Real :: SPD,Pi    

    DOUBLE PRECISION :: normal_size
    DOUBLE PRECISION :: Potential
    DOUBLE PRECISION :: Half_Space_Potential
    DOUBLE PRECISION :: Anomaly_Potential

    Integer, Allocatable, Dimension(:) :: IPIV  
    DOUBLE PRECISION, Allocatable, Dimension(:) :: RHS
    DOUBLE PRECISION, Allocatable, Dimension(:,:) :: Local_LHS
    DOUBLE PRECISION, Allocatable, Dimension(:) :: Cell_Electric_Charge
    Real, Allocatable, Dimension(:) :: PEL
    DOUBLE PRECISION, Allocatable, Dimension(:) :: L_Pole_Pole_DC
    DOUBLE PRECISION, Allocatable, Dimension(:) :: Apparent_resistivity    

    Pi = 3.141592
 
    Allocate(RHS(Number_of_Cells))
    Allocate(Local_LHS(Number_of_Cells,Number_of_Cells))
    Allocate(IPIV(Number_of_Cells))
    Allocate(Cell_Electric_Charge(Number_of_Cells))
    Allocate(PEL(3))
    Allocate(L_Pole_Pole_DC(251))
    Allocate(Apparent_resistivity(251))   
!==================================================================================================================  

    ! Producing the right hand side of the final matrix equation which will
    ! be soleved for electrical charge node coefficients

    Call Normal_Gradient_Source_Potential(E_CON,A_CON,current,Number_of_Cells,SL,ele_normal,COG,RHS)

    !Open(40,file='LNP_RHS.txt')
    !Do i=1,Number_of_Cells
    !    Write(40,*)   i, "   ", RHS(i)
    !End Do

!==================================================================================================================
    ! Producing the Left hand side of the final matrix equation will
    ! be soleved for electrical charge node coefficients

    Call Local_Coefficient_Matrix(E_CON,A_CON,integral_order,Number_of_Nodes,Number_of_Cells,&
        &Cell_Node_Numbers,ele_normal,Node_coordinates,COG,Local_LHS)
        !open(45,file='LNP_LHS')
        !Do i=1,Number_of_Cells
            !Write(45,*) "LHS(",i,",","1) : ","   ",Local_LHS(i,1)
            !Write(45,*) "LHS(",i,",","2) : ","   ",Local_LHS(i,2)
            !Write(45,*) "LHS(",i,",","3) : ","   ",Local_LHS(i,3)
            !Write(45,*) "LHS(",i,",","4) : ","   ",Local_LHS(i,4)
            !Write(45,*) "LHS(",i,",","5) : ","   ",Local_LHS(i,5)
            !Write(45,*) "LHS(",i,",","6) : ","   ",Local_LHS(i,6)
            !Write(45,*) "LHS(",i,",","7) : ","   ",Local_LHS(i,7)
            !Write(45,*) "LHS(",i,",","8) : ","   ",Local_LHS(i,8)
            !Write(45,*) "LHS(",i,",","9) : ","   ",Local_LHS(i,9)
            !Write(45,*) "LHS(",i,",","10) : ","   ",Local_LHS(i,10)
            !Write(45,*) "LHS(",i,",","11) : ","   ",Local_LHS(i,11)
            !Write(45,*) "LHS(",i,",","12) : ","   ",Local_LHS(i,12)
            !Write(45,*) "LHS(",i,",","13) : ","   ",Local_LHS(i,13)
            !Write(45,*) "LHS(",i,",","14) : ","   ",Local_LHS(i,14)
            !Write(45,*) "LHS(",i,",","15) : ","   ",Local_LHS(i,15)
            !Write(45,*) "LHS(",i,",","16) : ","   ",Local_LHS(i,16)
            !Write(45,*) "================================"
        !End Do


!==================================================================================================================
   ! Solving the final matrix equation to produce the electrical charge node coefficients

    NRHS = 1
    LDA = Number_of_Cells
    LDB = Number_of_Cells

     Call dgesv(Number_of_Cells,NRHS, Local_LHS, LDA, IPIV, RHS, LDB, INFO)
     If (INFO/=0) Then
         Print*,'stop : INFO/=0',INFO
         Stop
     End If
     Do i=1,Number_of_Cells
         Cell_Electric_Charge(i) = RHS(i)
     End Do
  

      Open(50,file='LNP_Cell_Charge.txt')
     Do i=1,Number_of_Cells
         Write(50,*)  i , "     ", Cell_Electric_Charge(i)
     End Do
!==================================================================================================================
     !Calculation of the DC electrical Potential

     Do i=1,251
        PEL(1) = Potential_Electrod(i,1)
        PEL(2) = Potential_Electrod(i,2)
        PEL(3) = Potential_Electrod(i,3)
    Call DC_Potential(E_CON,current,integral_order,Cell_Electric_Charge,Number_of_Nodes,Number_of_Cells,&
    &SL,PEL,Node_coordinates,Cell_Node_Numbers,ele_normal,Half_Space_Potential,Anomaly_Potential,Potential)
  
    L_Pole_Pole_DC(i) = Anomaly_Potential
    SPD = ((SL(1) - PEL(1))**2 + (SL(2) - PEL(2))**2 + (SL(3) - PEL(3))**2)**(0.5)
    Apparent_resistivity(i) = ((2 * Pi * SPD * Potential) / (current))

    End Do

    Open(55,file='L_Pole_Pole_DC.txt')
    k=0
    Do i=1,251
    Write(55,*)   k,"  ",L_Pole_Pole_DC(i)
    k = k+10
    End Do

    Open(181,file='resistivity.txt')
    k=0
    Do i=1,251
    Write(181,*)   k,"  ",Apparent_resistivity(i)/(10000)
    k = k+10
    End Do

End Subroutine Local_Normal_Potential