SUBROUTINE DC_Potential(E_CON,current,integral_order,Node_Coefficient_Electric_Charge,Number_of_Nodes,Number_of_Cells,&
    &SL,PEL,Node_coordinates,Cell_Node_Numbers,CCN,ele_normal,Half_Space_Potential,Anomaly_Potential,Potential)

    Implicit None

!==================================================================================================================
    ! Subroutine for calculation of the transformed Shape(Basis) Function
    INTERFACE
    SUBROUTINE Basis_Functions_Triangle(Node,Cellindex,ele_normal,BF)
      DOUBLE PRECISION, Allocatable, Dimension(:,:), INTENT(IN) :: Node
      DOUBLE PRECISION, Allocatable, Dimension(:,:), INTENT(IN) :: ele_normal
      DOUBLE PRECISION, Allocatable, Dimension(:), INTENT(OUT) :: BF
      Integer, INTENT(IN) :: Cellindex

     ! BF : Basis Function
    END SUBROUTINE Basis_Functions_Triangle
    END INTERFACE
!==================================================================================================================
    ! Subroutine for numerical calculation of the anomaly's potential integral
    INTERFACE
    Subroutine Anomaly_Potential_Integral(Node,BF,Cellindex,ele_normal,xi,yi,zi,integral_order,Int)
      DOUBLE PRECISION, Allocatable, Dimension(:,:), INTENT(IN) :: Node
      DOUBLE PRECISION, Allocatable, Dimension(:), INTENT(IN) :: BF
      DOUBLE PRECISION, Allocatable, Dimension(:,:), INTENT(IN) :: ele_normal
      Integer, INTENT(IN) :: integral_order,Cellindex
      Real, INTENT(IN) :: xi,yi,zi
      DOUBLE PRECISION, INTENT(OUT) :: Int
    END SUBROUTINE Anomaly_Potential_Integral
    END INTERFACE
!==================================================================================================================
      Integer, Allocatable, Dimension(:,:), INTENT(IN) :: Cell_Node_Numbers
      DOUBLE PRECISION, Allocatable, Dimension(:,:), INTENT(IN) :: Node_coordinates
      Integer, Allocatable, Dimension(:,:), INTENT(IN) :: CCN
      DOUBLE PRECISION, Allocatable, Dimension(:), INTENT(IN) :: Node_Coefficient_Electric_Charge
      Real, Allocatable, Dimension(:), INTENT(IN) :: SL
      Real, Allocatable, Dimension(:), INTENT(IN) :: PEL
      DOUBLE PRECISION, Allocatable, Dimension(:,:), INTENT(IN) :: ele_normal
      Integer, INTENT(IN) :: Number_of_Nodes,Number_of_Cells,integral_order
      Real, INTENT(IN) :: E_CON,current
      DOUBLE PRECISION, INTENT(OUT) :: Potential
      DOUBLE PRECISION, INTENT(OUT) :: Half_Space_Potential
      DOUBLE PRECISION, INTENT(OUT) :: Anomaly_Potential
!==================================================================================================================
    Integer :: i,j,m,k,l,Cellindex,m1
     Real :: C,Pi,Epsilon0,constant
     DOUBLE PRECISION :: Anomaly_Potential
     Integer :: a1,a2
     DOUBLE PRECISION :: co1,co2,Int
     Real :: xi,yi,zi

    DOUBLE PRECISION, Allocatable, Dimension(:,:) :: Node
    DOUBLE PRECISION, Allocatable, Dimension(:) :: BF




    Allocate(Node(3,4))
    Allocate(BF(4))
   

    Do i=1,3
        Do j=1,4
            Node(i,j) = 0.0
        End Do
    End Do
    Do i=1,4
        BF(i) = 0.0
    End Do

    Pi = 3.141592
    Epsilon0 = 8.854187817620
    
     constant = (1.E12)/(4* pi * Epsilon0)
    
!-----------------------------------------------------------------------------------------------------------------------
    ! Half_Space_Potential : Green_Function = (I/4*Pi*E_Con)* 1/((x-xs)^2+(y-ys)^2+(z-zs)^2)^(0.5)

    C = current / (4 * Pi * E_CON)
    
    Half_Space_Potential = (C / (((PEL(1)-SL(1))**2+(PEL(2)-SL(2))**2+(PEL(3)-SL(3))**2)**(0.5)))&
    +(C / (((PEL(1)-SL(1))**2+(PEL(2)-SL(2))**2+(PEL(3)+SL(3))**2)**(0.5)))
!-----------------------------------------------------------------------------------------------------------------------
!  Calculation of the potential due to the anomaly

        co2 = 0
        Do j=1,Number_of_Nodes
            co1 = 0
            Do l=2,Number_of_Cells+1
                a1 = CCN(j,l)
                If (a1.ne.0) Then
                m = 1
                Do k=2,4
                    a2 = Cell_Node_Numbers(a1,k)
                    If(a2.eq.j) Then
                        Node(1,1) = 1
                        Node(1,2) = Node_coordinates(j,2)
                        Node(1,3) = Node_coordinates(j,3)
                        Node(1,4) = Node_coordinates(j,4)
                    Else
                        m = m + 1
                        Node(m,1) = m
                        Node(m,2) = Node_coordinates(a2,2)
                        Node(m,3) = Node_coordinates(a2,3)
                        Node(m,4) = Node_coordinates(a2,4)
                    End If
                End Do

                Cellindex = a1
                Call Basis_Functions_Triangle(Node,Cellindex,ele_normal,BF)

                xi = PEL(1)
                yi = PEL(2)
                zi = PEL(3)

                Call Anomaly_Potential_Integral(Node,BF,Cellindex,ele_normal,xi,yi,zi,integral_order,Int)
                co1 = co1 + Int
                !Print*, " Co1 = ", co1
                End If
            End Do            ! 
            co2 = co2 + Node_Coefficient_Electric_Charge(j) * co1
            !Print*, " Co2 = ", co2
        End Do
        
        Anomaly_Potential = constant * co2
        Potential = Half_Space_Potential + Anomaly_Potential

End Subroutine