Subroutine Local_Normal_Potential_Cell_Based(Number_of_Nodes,Number_of_Cells,CCN,ele_normal,&
    &E_CON,A_CON,current,Node_coordinates,SL,integral_order,Cell_Node_Numbers,Potential_Electrod,normal_vector)

    Implicit None

!==================================================================================================================
    ! Subroutine for Computation of the normal gradient of source potential (Right hand side of the final matrix equation)
    INTERFACE
    SUBROUTINE Normal_Gradient_Source_Potential(E_CON,A_CON,current,Number_of_Nodes,SL,normal_vector,Node_coordinates,RHS)
      Real, Allocatable, Dimension(:), INTENT(IN) :: SL
      DOUBLE PRECISION, Allocatable, Dimension(:,:), INTENT(IN) :: normal_vector
      DOUBLE PRECISION, Allocatable, Dimension(:,:), INTENT(IN) :: Node_coordinates
      DOUBLE PRECISION, Allocatable, Dimension(:), INTENT(OUT) :: RHS
     ! RHS : Right Hand Side Matrix
     ! SL : Location of Electric Current Source
     Integer, INTENT(IN) :: Number_of_Nodes
      Real, INTENT(IN) :: E_CON,A_CON,current
    END SUBROUTINE Normal_Gradient_Source_Potential
    END INTERFACE
!==================================================================================================================
    ! Subroutine for Computation of the Left hand side of the final matrix equation
    INTERFACE
    SUBROUTINE Local_Coefficient_Matrix(E_CON,A_CON,integral_order,Number_of_Nodes,Number_of_Cells,CCN,&
        &Cell_Node_Numbers,ele_normal,normal_vector,Node_coordinates,Local_LHS)
      Integer, Allocatable, Dimension(:,:), INTENT(IN) :: Cell_Node_Numbers
      DOUBLE PRECISION, Allocatable, Dimension(:,:), INTENT(IN) :: Node_coordinates
      Integer, Allocatable, Dimension(:,:), INTENT(IN) :: CCN
      DOUBLE PRECISION, Allocatable, Dimension(:,:), INTENT(IN) :: ele_normal
      DOUBLE PRECISION, Allocatable, Dimension(:,:), INTENT(IN) :: normal_vector
      DOUBLE PRECISION, Allocatable, Dimension(:,:), INTENT(OUT) :: Local_LHS
     ! LHS : Left Hand Side Matrix (Coessicien Matrix in LHS C = RHS)
     Integer, INTENT(IN) :: Number_of_Nodes,Number_of_Cells,integral_order
      Real, INTENT(IN) :: E_CON,A_CON
    END SUBROUTINE Local_Coefficient_Matrix
    END INTERFACE
!==================================================================================================================
    !Subroutine for Computation of the DC potential
    INTERFACE
    SUBROUTINE Potential_Cell_Based(E_CON,current,integral_order,Node_Coefficient_Electric_Charge,Number_of_Nodes,Number_of_Cells,&
    &SL,PEL,Node_coordinates,Cell_Node_Numbers,ele_normal,Half_Space_Potential,Anomaly_Potential,Potential)

    !DCP_ : DC Potential, The first letters of the subroutine's name
      Integer, Allocatable, Dimension(:,:), INTENT(IN) :: Cell_Node_Numbers
      DOUBLE PRECISION, Allocatable, Dimension(:,:), INTENT(IN) :: Node_coordinates      
      DOUBLE PRECISION, Allocatable, Dimension(:), INTENT(IN) :: Node_Coefficient_Electric_Charge
      Real, Allocatable, Dimension(:), INTENT(IN) :: SL
      Real, Allocatable, Dimension(:), INTENT(IN) :: PEL
      DOUBLE PRECISION, Allocatable, Dimension(:,:), INTENT(IN) :: ele_normal
      Integer, INTENT(IN) :: Number_of_Nodes,Number_of_Cells,integral_order
      Real, INTENT(IN) :: E_CON,current
      DOUBLE PRECISION, INTENT(OUT) :: Potential
      DOUBLE PRECISION, INTENT(OUT) :: Half_Space_Potential
      DOUBLE PRECISION, INTENT(OUT) :: Anomaly_Potential
    END SUBROUTINE Potential_Cell_Based
    END INTERFACE
!==================================================================================================================
      Integer, Allocatable, Dimension(:,:), INTENT(IN) :: CCN
      Integer, Allocatable, Dimension(:,:), INTENT(IN) :: Cell_Node_Numbers
      DOUBLE PRECISION, Allocatable, Dimension(:,:), INTENT(IN) :: ele_normal
      DOUBLE PRECISION, Allocatable, Dimension(:,:), INTENT(IN) :: Node_coordinates
      Real, Allocatable, Dimension(:), INTENT(IN) :: SL
      Real, Allocatable, Dimension(:,:), INTENT(IN) :: Potential_Electrod
      Integer, INTENT(IN) :: Number_of_Nodes,Number_of_Cells,integral_order
      Real, INTENT(IN) :: E_CON,A_CON,current
      DOUBLE PRECISION, Allocatable, Dimension(:,:), INTENT(IN) :: normal_vector

!==================================================================================================================
    Integer :: ignp1
    DOUBLE PRECISION :: rgnp1,rgnp2,rgnp3,rgnp4
    ! ignp :: integer variable in the subroutine Global Normal Potential
    ! rgnp :: real variable in the subroutine Global Normal Potential
    Integer :: i,j,l,k
    Integer :: NRHS,LDA,LDB,INFO
    Real :: SPD,Pi

    DOUBLE PRECISION :: normal_size
    DOUBLE PRECISION :: Potential
    DOUBLE PRECISION :: Half_Space_Potential
    DOUBLE PRECISION :: Anomaly_Potential

    Integer, Allocatable, Dimension(:) :: IPIV
    DOUBLE PRECISION, Allocatable, Dimension(:) :: RHS
    DOUBLE PRECISION, Allocatable, Dimension(:,:) :: Local_LHS
    DOUBLE PRECISION, Allocatable, Dimension(:) :: Node_Coefficient_Electric_Charge
    Real, Allocatable, Dimension(:) :: PEL
    DOUBLE PRECISION, Allocatable, Dimension(:) :: L_Pole_Pole_DC
    DOUBLE PRECISION, Allocatable, Dimension(:) :: Apparent_resistivity

    Pi = 3.141592

    Allocate(RHS(Number_of_Nodes))
    Allocate(Local_LHS(Number_of_Nodes,Number_of_Nodes))
    Allocate(IPIV(Number_of_Nodes))
    Allocate(Node_Coefficient_Electric_Charge(Number_of_Nodes))
    Allocate(PEL(3))
    Allocate(L_Pole_Pole_DC(251))
    Allocate(Apparent_resistivity(251))
!==================================================================================================================

    ! Producing the right hand side of the final matrix equation which will
    ! be soleved for electrical charge node coefficients

    Call Normal_Gradient_Source_Potential(E_CON,A_CON,current,Number_of_Nodes,SL,normal_vector,Node_coordinates,RHS)

!==================================================================================================================
    ! Producing the Left hand side of the final matrix equation will
    ! be soleved for electrical charge node coefficients

    Call Local_Coefficient_Matrix(E_CON,A_CON,integral_order,Number_of_Nodes,Number_of_Cells,CCN,&
        &Cell_Node_Numbers,ele_normal,normal_vector,Node_coordinates,Local_LHS)

!==================================================================================================================
   ! Solving the final matrix equation to produce the electrical charge node coefficients

    NRHS = 1
    LDA = Number_of_Nodes
    LDB = Number_of_Nodes
     Call dgesv(Number_of_Nodes,NRHS, Local_LHS, LDA, IPIV, RHS, LDB, INFO)
     If (INFO/=0) Then
         Print*,'stop : INFO/=0',INFO
         Stop
     End If
     Do i=1,Number_of_Nodes
         Node_Coefficient_Electric_Charge(i) = RHS(i)
     End Do
!==================================================================================================================
     !Calculation of the DC electrical Potential

     Do i=1,251
        PEL(1) = Potential_Electrod(i,1)
        PEL(2) = Potential_Electrod(i,2)
        PEL(3) = Potential_Electrod(i,3)
    Call Potential_Cell_Based(E_CON,current,integral_order,Node_Coefficient_Electric_Charge,Number_of_Nodes,Number_of_Cells,&
    &SL,PEL,Node_coordinates,Cell_Node_Numbers,ele_normal,Half_Space_Potential,Anomaly_Potential,Potential)

    L_Pole_Pole_DC(i) = Anomaly_Potential
    SPD = ((SL(1) - PEL(1))**2 + (SL(2) - PEL(2))**2 + (SL(3) - PEL(3))**2)**(0.5)
    Apparent_resistivity(i) = ((2 * Pi * SPD * Potential) / (current))

    End Do

    Open(555,file='LC_Pole_Pole_DC.txt')
    k=0
    Do i=1,251
    Write(555,*)   k,"  ",L_Pole_Pole_DC(i)
    k = k+10
    End Do

    !Open(181,file='resistivity.txt')
    !k=0
    !Do i=1,251
    !Write(181,*)   k,"  ",Apparent_resistivity(i)/(10000)
    !k = k+10
    !End Do

End Subroutine Local_Normal_Potential_Cell_Based
