SUBROUTINE Potential_Cell_Based(E_CON,current,integral_order,Node_Coefficient_Electric_Charge,Number_of_Nodes,Number_of_Cells,&
    &SL,PEL,Node_coordinates,Cell_Node_Numbers,ele_normal,Half_Space_Potential,Anomaly_Potential,Potential)

    Implicit None

!==================================================================================================================
    ! Subroutine for calculation of the transformed Shape(Basis) Function
    INTERFACE
    SUBROUTINE Basis_Functions_Cell(Node,Cellindex,ele_normal,BFC)
      DOUBLE PRECISION, Allocatable, Dimension(:,:), INTENT(IN) :: Node
      DOUBLE PRECISION, Allocatable, Dimension(:,:), INTENT(IN) :: ele_normal
      DOUBLE PRECISION, Allocatable, Dimension(:,:), INTENT(OUT) :: BFC
      Integer, INTENT(IN) :: Cellindex

     ! BFC : Basis Function Cell Based
    END SUBROUTINE Basis_Functions_Cell
    END INTERFACE
!==================================================================================================================
    ! Subroutine for numerical calculation of the anomaly's potential integral
    INTERFACE
    Subroutine Anomaly_Potential_Cell_Based(Node,BFC,Cellindex,ele_normal,xi,yi,zi,integral_order,EC,Int)
      DOUBLE PRECISION, Allocatable, Dimension(:,:), INTENT(IN) :: Node
      DOUBLE PRECISION, Allocatable, Dimension(:,:), INTENT(IN) :: BFC
      DOUBLE PRECISION, Allocatable, Dimension(:,:), INTENT(IN) :: ele_normal
      DOUBLE PRECISION, Allocatable, Dimension(:), INTENT(IN)   :: EC
      Integer, INTENT(IN) :: integral_order,Cellindex
      Real, INTENT(IN) :: xi,yi,zi
      DOUBLE PRECISION, INTENT(OUT) :: Int
    END SUBROUTINE Anomaly_Potential_Cell_Based
    END INTERFACE
!==================================================================================================================
      Integer, Allocatable, Dimension(:,:), INTENT(IN) :: Cell_Node_Numbers
      DOUBLE PRECISION, Allocatable, Dimension(:,:), INTENT(IN) :: Node_coordinates
      DOUBLE PRECISION, Allocatable, Dimension(:), INTENT(IN) :: Node_Coefficient_Electric_Charge
      Real, Allocatable, Dimension(:), INTENT(IN) :: SL
      Real, Allocatable, Dimension(:), INTENT(IN) :: PEL
      DOUBLE PRECISION, Allocatable, Dimension(:,:), INTENT(IN) :: ele_normal
      Integer, INTENT(IN) :: Number_of_Nodes,Number_of_Cells,integral_order
      Real, INTENT(IN) :: E_CON,current
      DOUBLE PRECISION, INTENT(OUT) :: Potential
      DOUBLE PRECISION, INTENT(OUT) :: Half_Space_Potential
      DOUBLE PRECISION, INTENT(OUT) :: Anomaly_Potential
!==================================================================================================================
    Integer :: i,j,m,k,l,Cellindex,m1
     Real :: C,Pi,Epsilon0,constant
     DOUBLE PRECISION :: Anomaly_Potential
     Integer :: a1,a2
     DOUBLE PRECISION :: co1,co2,Int
     Real :: xi,yi,zi

    DOUBLE PRECISION, Allocatable, Dimension(:) :: EC
    DOUBLE PRECISION, Allocatable, Dimension(:,:) :: Node
    DOUBLE PRECISION, Allocatable, Dimension(:,:) :: BFC
    


    Allocate(Node(3,4))
    Allocate(BFC(3,5))
    Allocate(EC(3))


    Do i=1,3
        Do j=1,4
            Node(i,j) = 0.0
        End Do
    End Do
    Do i=1,3
      Do j=1,5
        BFC(i,j) = 0.0
      End Do
    End Do

    Pi = 3.141592
    Epsilon0 = 8.854187817620
    constant = (1.E12)/(4* pi * Epsilon0)

!-----------------------------------------------------------------------------------------------------------------------
    ! Half_Space_Potential : Green_Function = (I/4*Pi*E_Con)* 1/((x-xs)^2+(y-ys)^2+(z-zs)^2)^(0.5)

    C = current / (4 * Pi * E_CON)

    Half_Space_Potential = (C / (((PEL(1)-SL(1))**2+(PEL(2)-SL(2))**2+(PEL(3)-SL(3))**2)**(0.5)))&
    +(C / (((PEL(1)-SL(1))**2+(PEL(2)-SL(2))**2+(PEL(3)+SL(3))**2)**(0.5)))
!-----------------------------------------------------------------------------------------------------------------------
!  Calculation of the potential due to the anomaly

        co1 = 0
        Do j=1,Number_of_Cells
                 m1 = 0
                Do k=2,4
                    a2 = Cell_Node_Numbers(j,k)
                    m1 = m1 + 1
                    Node(m1,1) = a2
                    Node(m1,2) = Node_coordinates(a2,2)
                    Node(m1,3) = Node_coordinates(a2,3)
                    Node(m1,4) = Node_coordinates(a2,4)
                    EC(m1) = Node_Coefficient_Electric_Charge(a2)
                End Do


                Cellindex = j

                Call Basis_Functions_Cell(Node,Cellindex,ele_normal,BFC)

                xi = PEL(1)
                yi = PEL(2)
                zi = PEL(3)

                Call Anomaly_Potential_Cell_Based(Node,BFC,Cellindex,ele_normal,xi,yi,zi,integral_order,EC,Int)

            co1 = co1 + Int
        End Do

        Anomaly_Potential = constant * co1
        Potential = Half_Space_Potential + Anomaly_Potential

End Subroutine Potential_Cell_Based